/* 
 * linux/arch/arm/boot/compressed/head-s3c2800.S
 * 
 * Samsung s3c2800 cpu
 *
 * Author: Brad Parker <brad@heeltoe.com>
 *
 * 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#include <linux/config.h>
#include <asm/mach-types.h>

		.section        ".start", #alloc, #execinstr

/* values for 200Mhz operation in PLLCON */
/* S3C2800; ext clk=10Mhz, fclk=203Mhz, pclk=50.75Mhz */
#define MDIV		0x5c
#define PDIV		0x3
#define SDIV		0x0
	
/* registers */
#define INTMOD		0x10020004
#define INTMSK		0x10020008
#define WTCON		0x10120004

#define PCONF		0x1010003c
#define PDATF		0x10100040
#define PUPF		0x10100044

#define PLLCON		0x10000000
#define CLKCON		0x10000004
#define LOCKTIME	0x1000000c
#define SMBCON0		0x10010004
#define SMBCON1		0x10010008
#define SMBCON2		0x1001000c
#define SMBCON3		0x10010010
#define	REFRESH		0x10010014
#define	DMTMCON		0x10010018
#define	MRSR		0x1001001c
		
/* clkcon bits */
#define CLKCON_PCLK	(1 << 12)
#define CLKCON_HCLK	(1 << 11)

/***************************************/


#define BOOT_PROM_MISSING		/* if boot prom is not programmed */
#define CONFIG_S3C2800_CAMELOT_LED	/* show progress on led's */

#define LED_LOOP_DELAY	0x00010000
			
__S3C2800_start:

	@ Preserve r8/r7 i.e. kernel entry values

#if defined(CONFIG_MACH_FUJITSU_CAMELOT)
	mov	r7, #(MACH_TYPE_FUJITSU_CAMELOT&0xff)
	orr	r7, r7, #(MACH_TYPE_FUJITSU_CAMELOT&0xff00)
	mov	r8, #0
#endif
			
	ldr	r0,=WTCON       /* disable watch dog reset */
	ldr	r1,=0xa5         
	str	r1,[r0]

	ldr	r0,=INTMSK
	ldr	r1,=0x0		/* mask all interrupts */
	str	r1,[r0]

	ldr     r0, =PCONF	/* init portf for leds */
	ldr     r1, =0x55aa
	str     r1, [r0]
	
	ldr     r0, =PUPF	/* disable pullups */
	ldr     r1, =0xff
	str     r1, [r0]

#if defined(CONFIG_S3C2800_CAMELOT_LED)
	ldr     r0, =PDATF
	ldr     r1, =0x10
	str     r1, [r0]
	mov	r0, #LED_LOOP_DELAY
1:	subs	r0, r0, #1
	bne	1b
#endif	
				
	/* h=f/2, p=h/2, turn all the clocks on */
	ldr	r0,=CLKCON
	ldr	r1,=(CLKCON_PCLK | CLKCON_HCLK | 0x7fc)
/*	ldr	r1,=0x1ffc*/
	str	r1,[r0]	
	
	/* to reduce PLL lock time, adjust the LOCKTIME register */
	ldr	r0,=LOCKTIME
	ldr	r1,=0xffffff
	str	r1,[r0]	

#if defined(CONFIG_S3C2800_CAMELOT_LED)
 	ldr     r0, =PDATF
	ldr     r1, =0xf0
	str     r1, [r0]
 	ldr     r0, =PDATF
	ldr     r1, =0x20
	str     r1, [r0]
	mov	r0, #LED_LOOP_DELAY
1:	subs	r0, r0, #1
	bne	1b
#endif

	/* configure PLL */
	ldr	r0,=PLLCON          
	ldr	r1,=((MDIV<<12)+(PDIV<<4)+SDIV)
	str	r1,[r0]
	
	/* some delay after setting pll */
	mov	r0,#8000
1:		
	subs	r0,r0,#1
	bne	1b	

#if defined(CONFIG_S3C2800_CAMELOT_LED)
 	ldr     r0, =PDATF
	ldr     r1, =0xf0
	str     r1, [r0]
 	ldr     r0, =PDATF
	ldr     r1, =0x30
	str     r1, [r0]
	mov	r0, #LED_LOOP_DELAY
1:	subs	r0, r0, #1
	bne	1b
#endif

	/*
	 * this code should be removed once a boot prom is in place which
	 * will setup the memory timing and chip selects...
	 */
#ifdef BOOT_PROM_MISSING
	/* boot prom should do this but we don't have one yet */
	ldr	r0,=SMBCON0
	ldr	r1,=0x0001589
	str	r1,[r0]	

	ldr	r0,=SMBCON1
	ldr	r1,=0x0001589
	str	r1,[r0]	

	ldr	r0,=SMBCON2
	ldr	r1,=0x00101582
	str	r1,[r0]	

	ldr	r0,=SMBCON3
	ldr	r1,=0x00101580
	str	r1,[r0]	
		
#if defined(CONFIG_S3C2800_CAMELOT_LED)
	ldr     r0, =PDATF
	ldr     r1, =0xff
	str     r1, [r0]
	ldr     r0, =PDATF
	ldr     r1, =0x40
	str     r1, [r0]
	mov	r0, #LED_LOOP_DELAY
1:	subs	r0, r0, #1
	bne	1b
#endif
			
	/* set up sdram timing (again, boot prom...) */
	ldr	r0,=REFRESH
	ldr	r1,=0x008201e9
	str	r1,[r0]	

	ldr	r0,=DMTMCON
	ldr	r1,=0x00010d10
	str	r1,[r0]	
	
	ldr	r0,=MRSR
	ldr	r1,=0x00000020
	str	r1,[r0]	
		
#if defined(CONFIG_S3C2800_CAMELOT_LED)
	ldr     r0, =PDATF
	ldr     r1, =0xff
	str     r1, [r0]
	ldr     r0, =PDATF
	ldr     r1, =0x50
	str     r1, [r0]
	mov	r0, #LED_LOOP_DELAY
1:	subs	r0, r0, #1
	bne	1b
#endif
	
#endif /* BOOT_PROM_MISSING */
			
	b	1f
	
	.pool		

	.align 8

1:

