/*
 * arch/arm/kernel/kgdb-16550.c
 *
 * Low level KGDB code for 16550 based UARTs
 *
 * Original IXP200 code by Naeem Afzal <naeem.m.afzal@intel.com>
 * Genercized by Deepak Saxena <dsaxena@mvista.com>
 *
 * Copyright (c) 2002 Intel Corp.
 * Copyrigh (c) 2003 MontaVista Software, Inc.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 */
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/serial_reg.h>

#include <asm/atomic.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/unistd.h>

#include <asm/kgdb.h>
#include <asm/serial.h>
#include <asm/hardware.h>
#include <asm/irq.h>


extern struct serial_state rs_table[];
extern int kgdb_ttyS;
extern int kgdb_baud;

static inline volatile char serial_in(int offset)
{
	return readb((unsigned long)rs_table[kgdb_ttyS].iomem_base + 
			(offset << rs_table[kgdb_ttyS].iomem_reg_shift));
}

static inline void serial_out(int offset, char value)
{
	writeb(value, (unsigned long)rs_table[kgdb_ttyS].iomem_base +
		       	(offset << rs_table[kgdb_ttyS].iomem_reg_shift));
}

void kgdb_serial_init(void)
{
	int quot = 0;

	if(kgdb_ttyS >= RS_TABLE_SIZE) {
		printk("KGDB: Serial port %d does not exist, defaulting to ttyS0\n", kgdb_ttyS);
		kgdb_ttyS = 0;
	}

	quot = rs_table[kgdb_ttyS].baud_base / kgdb_baud;

	/*
	 * If this is an early breakpoint (built-in) before rs_init(),
	 * we need disable autoconf or it will kill KGDB communication!
	 */
	rs_table[kgdb_ttyS].flags &= ~ASYNC_BOOT_AUTOCONF;

	serial_out(UART_LCR, UART_LCR_DLAB); /* enable access to divisor reg */
	serial_out(UART_DLL, quot & 0xff); 
	serial_out(UART_DLM, quot >> 8); 
	serial_out(UART_LCR, UART_LCR_WLEN8);
	serial_out(UART_FCR, 
		UART_FCR_ENABLE_FIFO|UART_FCR_CLEAR_RCVR|UART_FCR_CLEAR_XMIT);

	/* 
	 * Should do nothing on non-Xscale 16550's 
	 * So far testing has proven this to be the case
	 */
	serial_out(UART_IER, UART_IER_UUE); 

	return;
}

void kgdb_serial_putchar(unsigned char ch)
{
	unsigned char status;

	do {
		status = serial_in(UART_LSR);
	} while ((status & (UART_LSR_TEMT|UART_LSR_THRE)) !=
			(UART_LSR_TEMT|UART_LSR_THRE));

	serial_out(0, ch);
}

unsigned char kgdb_serial_getchar(void)
{
	while(!(serial_in(UART_LSR) & UART_LSR_DR));
	
	return serial_in(UART_RX);
}

