/*
 *  linux/arch/arm/mach-pxa/lubbock.c
 *
 *  Support for the Intel DBPXA250 Development Platform.
 *  
 *  Author:	Nicolas Pitre
 *  Created:	Jun 15, 2001
 *  Copyright:	MontaVista Software Inc.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */
#include <linux/init.h>

#include <linux/major.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <asm/types.h>
#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>
#include <asm/arch/lubbock.h>

#include <asm/arch/irq.h>

#include "generic.h"

unsigned long cpld_mask1,cpld_mask2,cpld_mask3;
unsigned int pcmcia_cd=0;
unsigned int cf_cd=0;

int pcastate[2];

static void lubbock_mask_irq(unsigned int irq)
{
	int lubbock_irq = (irq - LUBBOCK_IRQ(0));

	if ( lubbock_irq >= 0 && lubbock_irq <= 7) {
		cpld_mask1 &= ~(1 << lubbock_irq);
		LUB_IRQ_GP1 = cpld_mask1;
	}
	if ( lubbock_irq >= 8 && lubbock_irq <= 15) {
		cpld_mask2 &= ~(1 << (lubbock_irq-8));
		LUB_IRQ_GP2 = cpld_mask2;
	}
	if ( lubbock_irq >= 16 && lubbock_irq <=23) {
		cpld_mask3 &= ~(1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3;
	}
}

static void lubbock_unmask_irq(unsigned int irq)
{
	
	int lubbock_irq = (irq - LUBBOCK_IRQ(0));

	if ( lubbock_irq >= 0 && lubbock_irq <= 7) {
		cpld_mask1 |= (1 << lubbock_irq);
		LUB_IRQ_GP1 = cpld_mask1;
	} else if ( lubbock_irq >= 8 && lubbock_irq <= 15) {
		cpld_mask2 |= (1 << (lubbock_irq-8));
		LUB_IRQ_GP2 = cpld_mask2;
	} else if ( lubbock_irq >= 16 && lubbock_irq <=19) {
		cpld_mask3 |= (1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3;
	} else if ( lubbock_irq == 20 ) {
		if ( !pcmcia_cd ) {
		cpld_mask3 |= (1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3; 
		} 
	} else if ( lubbock_irq == 21 ) {
		if ( pcmcia_cd ) {
		cpld_mask3 |= (1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3;
		} 
	} else if ( lubbock_irq == 22 ) {
		if ( !cf_cd ) {
		cpld_mask3 |= (1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3;
		} 
	} else if ( lubbock_irq == 23 ) {
		if ( cf_cd ) {
		cpld_mask3 |= (1 << (lubbock_irq-16));
		LUB_IRQ_GP3 = cpld_mask3;
		}
	}

}

void lubbock_irq_demux(int irq, void *dev_id, struct pt_regs *regs)
{
	unsigned long irq_status;
	int i;
	extern int pcastate[];

	while ((irq_status = LUB_IRQ_GP1 & cpld_mask1)) {
		for (i = 0; i < 6; i++) {
			if(irq_status & (1<<i)) 
				do_IRQ(LUBBOCK_IRQ(i), regs);
		}
	}
	while ((irq_status = LUB_IRQ_GP2 & cpld_mask2)) {
		for (i = 0; i < 6; i++) {
			if(irq_status & (1<<i)) 
				do_IRQ(LUBBOCK_IRQ(i+8), regs);
		} 
	}
	while ((irq_status = LUB_IRQ_GP3 & cpld_mask3)) {
		for (i = 0; i < 8; i++) {
			if(irq_status & (1<<i)) {
				if (i == 4) { 
				  lubbock_mask_irq(LUBBOCK_IRQ(i+16));
				  pcmcia_cd = 1;
                                  pcastate[0] = 1;
				  lubbock_unmask_irq(LUBBOCK_IRQ(i+16)+1);
				} else if (i == 5 ) {
				  lubbock_mask_irq(LUBBOCK_IRQ(i+16));
				  pcmcia_cd = 0;
				  pcastate[0] = 0;
				  lubbock_unmask_irq(LUBBOCK_IRQ(i+16)-1);
				} else if (i == 6 ) {
				  lubbock_mask_irq(LUBBOCK_IRQ(i+16));
				  cf_cd = 1;
				  pcastate[1] = 1;
				  lubbock_unmask_irq(LUBBOCK_IRQ(i+16)+1);
				} else if (i == 7 ) {
				  lubbock_mask_irq(LUBBOCK_IRQ(i+16));
				  cf_cd = 0;
				  pcastate[1] = 0;
				  lubbock_unmask_irq(LUBBOCK_IRQ(i+16)-1);
				} 
				do_IRQ(LUBBOCK_IRQ(i+16), regs);
			}
		}
	}
}

static struct irqaction lubbock_irq = {
	name:		"ICPDAS CPLD",
	handler:	lubbock_irq_demux,
	flags:		SA_INTERRUPT
};

static void __init lubbock_init_irq(void)
{
	int irq;
	
	pxa_init_irq();

	/* setup extra lubbock irqs */
	for(irq = LUBBOCK_IRQ(0); irq <= LUBBOCK_IRQ(23); irq++)
	{
		irq_desc[irq].valid	= 1;
		irq_desc[irq].probe_ok	= 1;
		irq_desc[irq].mask_ack	= lubbock_mask_irq;
		irq_desc[irq].mask	= lubbock_mask_irq;
		irq_desc[irq].unmask	= lubbock_unmask_irq;
	}

	set_GPIO_IRQ_edge(GPIO_LUBBOCK_IRQ, GPIO_RISING_EDGE);
	setup_arm_irq(IRQ_GPIO_LUBBOCK_IRQ, &lubbock_irq);

}

static int __init lubbock_init(void)
{
	printk(" Version ID = %x\n",LUB_VID);
	
	LUB_IRQ_GP1 = LUB_IRQ_GP2 = 0x00;
	LUB_IRQ_GP3 = 0x00;
	cpld_mask1=cpld_mask2=cpld_mask3 =0x00;
	return 0;
}

__initcall(lubbock_init);

static void __init
fixup_lubbock(struct machine_desc *desc, struct param_struct *params,
		char **cmdline, struct meminfo *mi)
{
	SET_BANK (0, 0xa0000000, 64*1024*1024);
	mi->nr_banks      = 1;
#if 0
	setup_ramdisk (1, 0, 0, 8192);
	setup_initrd (__phys_to_virt(0xa1000000), 4*1024*1024);
	ROOT_DEV = MKDEV(RAMDISK_MAJOR,0);
#endif
}

static struct map_desc lubbock_io_desc[] __initdata = {
 /* virtual     physical    length      domain     r  w  c  b */
  { 0xf0000000, 0x0c400000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* CPLD */
  { 0xf0100000, 0x0c800000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* CPLD */
  { 0xf0200000, 0x0cc00000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* CPLD */
  { 0xf0300000, 0x0d000000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* VID */
  { 0xf2800000, 0x13800000, 0x00200000, DOMAIN_IO, 0, 1, 0, 0 }, /* MQ200 */
  { 0xf2E00000, 0x13E00000, 0x00200000, DOMAIN_IO, 0, 1, 0, 0 }, /* MQ200 */
  { 0xf1000000, 0x0c000000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* LAN91C96 IO */
  { 0xf1100000, 0x0c200000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* LAN91C96 Attr */
  { 0xf4100000, 0x14200000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* USB */
  { 0xf4000000, 0x14000000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* 87K IO */
  { 0xf4200000, 0x14400000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 1 */
  { 0xf4300000, 0x14600000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 2 */
  { 0xf4400000, 0x14800000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 3 */
  { 0xf4500000, 0x14A00000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 4 */
  { 0xf4600000, 0x14C00000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 5 */
  { 0xf4700000, 0x14E00000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 6 */
  { 0xf4800000, 0x15000000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 7 */
  { 0xf4900000, 0x15200000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* UART 8 */
  { 0xf4A00000, 0x15600000, 0x00100000, DOMAIN_IO, 0, 1, 0, 0 }, /* CANBus */
  LAST_DESC
};

static void __init lubbock_map_io(void)
{
	pxa_map_io();
	iotable_init(lubbock_io_desc);

	/* This enables the BTUART */
	CKEN |= CKEN7_BTUART;
	set_GPIO_mode(GPIO42_BTRXD_MD);
	set_GPIO_mode(GPIO43_BTTXD_MD);
	set_GPIO_mode(GPIO44_BTCTS_MD);
	set_GPIO_mode(GPIO45_BTRTS_MD);

	/* This is for the SMC chip select */
	set_GPIO_mode(GPIO79_nCS_3_MD);

	/*   MECR 
	  MECR =0x03; */
	/* setup sleep mode values */
	PWER  = 0x00000002;
	PFER  = 0x00000000;
	PRER  = 0x00000002;
	PGSR0 = 0x00008000;
	PGSR1 = 0x003F0202;
	PGSR2 = 0x0001C000;
	PCFR |= PCFR_OPDE;
}

MACHINE_START(LUBBOCK, "ICPDAS SBC Platform")
	MAINTAINER("Austin.")
	BOOT_MEM(0xa0000000, 0x40000000, io_p2v(0x40000000))
	FIXUP(fixup_lubbock)
	MAPIO(lubbock_map_io)
	INITIRQ(lubbock_init_irq)
MACHINE_END
