/*
 * linux/drivers/pcmcia/pxa/pxa_lubbock.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Copyright (c) 2002 Accelent Systems, Inc. All Rights Reserved
 * 
 * Platform specific routines for the Accelent PXA250 IDP, based on those
 * first done for the Lubbock.
 *
 * Version 1.0 2002-05-02  Jeff Sutherland <jeffs@accelent.com>
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>

#include <pcmcia/ss.h>

#include <asm/delay.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/arch/pcmcia.h>
#include <asm/arch/lubbock.h>

static int 
pxa_lubbock_pcmcia_init(struct pcmcia_init *init)
{
	int return_val = 0;

  	set_GPIO_mode(81 | GPIO_OUT);   /* For Reset Pin */
	set_GPIO_mode(82 | GPIO_OUT);

	/* irq's for slots: */
	GPDR(IRQ_TO_GPIO_2_80(PCMCIA_S0_RDYINT)) &= ~GPIO_bit(IRQ_TO_GPIO_2_80(PCMCIA_S0_RDYINT));
	GPDR(IRQ_TO_GPIO_2_80(CF_S1_RDYINT)) &= ~GPIO_bit(IRQ_TO_GPIO_2_80(CF_S1_RDYINT));
	set_GPIO_IRQ_edge(IRQ_TO_GPIO_2_80(PCMCIA_S0_RDYINT), GPIO_RISING_EDGE);
	set_GPIO_IRQ_edge(IRQ_TO_GPIO_2_80(CF_S1_RDYINT), GPIO_RISING_EDGE);

	/* For Socket 0 input   */ 
	return_val += request_irq(LUBBOCK_PCMCIA_CD_IN_IRQ, init->handler, SA_INTERRUPT, "PXA PCMCIA CD0 IN", NULL);   

        /* For Socket 0 output   */ 
	return_val = request_irq(LUBBOCK_PCMCIA_CD_OUT_IRQ, init->handler, SA_INTERRUPT, "PXA PCMCIA CD0 OUT", NULL);     

	/* For Socket 1 input 	*/
	return_val += request_irq(LUBBOCK_CF_CD_IN_IRQ, init->handler, SA_INTERRUPT, "PXA CF CD1 IN", NULL);

	/* For Socket 1 output   */
	return_val += request_irq(LUBBOCK_CF_CD_OUT_IRQ, init->handler, SA_INTERRUPT, "PXA CF CD1 OUT", NULL);  
	if (return_val < 0) {
		free_irq(LUBBOCK_CF_CD_IN_IRQ, NULL);
		return -1;
	}

        return  2; /* Two Slot */
}

static int
pxa_lubbock_pcmcia_shutdown(void)
{
	free_irq(LUBBOCK_PCMCIA_CD_IN_IRQ, NULL);
	free_irq(LUBBOCK_PCMCIA_CD_OUT_IRQ, NULL);
	free_irq(LUBBOCK_CF_CD_IN_IRQ, NULL);
	free_irq(LUBBOCK_CF_CD_OUT_IRQ, NULL);
  	
	return 0;
}

static int
pxa_lubbock_pcmcia_socket_state(struct pcmcia_state_array *state_array)
{
	int return_val = 1;
	int i;
	extern int pcastate[];

	memset(state_array->state, 0,
	       (state_array->size) * sizeof (struct pcmcia_state));
	
	for (i = 0; i < 2; i++) {
	      	/* this one is a gpio */
		state_array->state[i].detect = pcastate[i] ;
		state_array->state[i].ready  = PCC_IRQ(i)?0:1;

		state_array->state[i].bvd1   =  1;
		state_array->state[i].bvd2   =  1;
		state_array->state[i].wrprot =  0;
		state_array->state[i].vs_3v  =  1;
		state_array->state[i].vs_Xv  =  1;
	}
	return return_val;
}

static int
pxa_lubbock_pcmcia_get_irq_info(struct pcmcia_irq_info *info)
{
	switch (info->sock) {
	    case 0:
		info->irq = PCMCIA_S0_RDYINT;
		break;
	    case 1:
		info->irq = CF_S1_RDYINT;
		break;
	    default:
		return -1;
	}
                                                                                                             		return 0;
}

static int
pxa_lubbock_pcmcia_configure_socket(unsigned int sock, socket_state_t *state)
{
       int i;

       switch (sock) {
	    case 0:
		if(state->flags & SS_RESET) {
		   GPSR2 |= 0x00020000;
		   i =100;	
		   while(i) i--;   	 	
		   GPCR2 |= 0x00020000; 
		}
	        break;
	    case 1:
		if(state->flags & SS_RESET) {                    		
		   GPSR2 |= 0x00040000;;
		   i =100;	
		   while(i) i--;   
                   GPCR2 |= 0x00040000; 
		}
	        break;
	}
	return 0;
}

struct pcmcia_low_level lubbock_pcmcia_ops = { 
  pxa_lubbock_pcmcia_init,
  pxa_lubbock_pcmcia_shutdown,
  pxa_lubbock_pcmcia_socket_state,
  pxa_lubbock_pcmcia_get_irq_info,
  pxa_lubbock_pcmcia_configure_socket
};
